/*
 * An opener for a "host" file provided via the C library.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "host_file_accessor.h"
#include "host_file_opener.h"

/* Return a file identifier for the given 'path'. */

fileid_t HostFileOpener::get_fileid(const char *path, flags_t flags)
{
    (void) flags;

    /* Test for a valid file or an attempt to create a file. */

    FILE *fp = fopen(path, "r");

    if (fp == NULL)
        return FILEID_INVALID;

    fclose(fp);

    return _get_fileid(path, true);
}

fileid_t HostFileOpener::_get_fileid(const char *path, bool create)
{
    std::lock_guard<std::mutex> guard(_lock);

    /* The inode number would be a good choice, but the L4Re read-only file
       implementation just provides the value of a counter!

       See: pkg/l4re-core/l4re_vfs/include/impl/ro_file_impl.h */

    std::string s(path);

    HostFileIdentifiers::iterator it = _fileids.find(s);

    if (it != _fileids.end())
        return it->second;

    if (!create)
        return FILEID_INVALID;

    fileid_t fileid = _fileids.size();

    _fileids[s] = fileid;

    return fileid;
}

/* Return a new accessor for 'fileid'. */

long HostFileOpener::make_accessor(const char *path, flags_t flags, fileid_t fileid, Accessor **accessor)
{
    // NOTE: Not testing for create or write flags.

    (void) flags;

    FILE *fp = fopen(path, "r");

    if (fp == NULL)
        return -L4_ENOENT;

    *accessor = new HostFileAccessor(fp, fileid);
    return L4_EOK;
}

// vim: tabstop=4 expandtab shiftwidth=4
